/*
 * Copyright (C) 2003 Advanced Software Engineering Limited.
 *
 * This file is part of the ChartDirector software. Usage of this file is
 * subjected to the ChartDirector license agreement. See the LICENSE.TXT
 * file that comes with the ChartDirector software release, or visit
 * Advanced Software Engineering's web site at www.advsofteng.com or
 * email to support@advsofteng.com for information.
 *
 */

#ifndef BCHARTDIR_HDR
#define BCHARTDIR_HDR

#include <wchar.h>


#ifdef _WIN32
#define CHARTDIR_DLLEXPORT __declspec(dllexport)
#else
#define CHARTDIR_DLLEXPORT
#endif

extern "C" {


////////////////////////////////////////////////////////////////////////////////////////
//	chartdir.h
////////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT int getVersion();
CHARTDIR_DLLEXPORT const char *getDescription();
CHARTDIR_DLLEXPORT const char *getCopyright();

CHARTDIR_DLLEXPORT bool testFont(const char *font, int fontIndex, double fontHeight,
		double fontWidth, double angle, char *buffer);

CHARTDIR_DLLEXPORT bool isLicensed();
CHARTDIR_DLLEXPORT bool getLicenseAttr(const char *key, char *buffer);
CHARTDIR_DLLEXPORT bool setLicenseFile(const char *filename, char *buffer);
CHARTDIR_DLLEXPORT bool setLicenseCode(const char *licCode, char *buffer);

CHARTDIR_DLLEXPORT double chartTime(int y, int m, int d, int h, int n, int s);
CHARTDIR_DLLEXPORT double chartTime2(int t);


////////////////////////////////////////////////////////////////////////////////////////
//	libgraphics.h
////////////////////////////////////////////////////////////////////////////////////////
class TTFTextInternal;
CHARTDIR_DLLEXPORT void CTTFText_destroy(TTFTextInternal *ptr);
CHARTDIR_DLLEXPORT int CTTFText_getWidth(TTFTextInternal *ptr);
CHARTDIR_DLLEXPORT int CTTFText_getHeight(TTFTextInternal *ptr);
CHARTDIR_DLLEXPORT int CTTFText_getLineHeight(TTFTextInternal *ptr);
CHARTDIR_DLLEXPORT int CTTFText_getLineDistance(TTFTextInternal *ptr);
CHARTDIR_DLLEXPORT void CTTFText_draw(TTFTextInternal *ptr, int x, int y, int color, int align);


class DrawAreaInternal;
CHARTDIR_DLLEXPORT DrawAreaInternal *CDrawArea_create();
CHARTDIR_DLLEXPORT void CDrawArea_destroy(DrawAreaInternal *ptr);
CHARTDIR_DLLEXPORT void CDrawArea_setSize(DrawAreaInternal *ptr, int width, int height, int bgColor);
CHARTDIR_DLLEXPORT void CDrawArea_resize(DrawAreaInternal *ptr, int newWidth, int newHeight, int f, double blur);
CHARTDIR_DLLEXPORT int CDrawArea_getWidth(DrawAreaInternal *ptr);
CHARTDIR_DLLEXPORT int CDrawArea_getHeight(DrawAreaInternal *ptr);
CHARTDIR_DLLEXPORT void CDrawArea_setClipRect(DrawAreaInternal *ptr, int left, int top, int right, int bottom);
CHARTDIR_DLLEXPORT void CDrawArea_setBgColor(DrawAreaInternal *ptr, int c);

CHARTDIR_DLLEXPORT void CDrawArea_move(DrawAreaInternal *ptr, double xOffset, double yOffset, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_rotate(DrawAreaInternal *ptr, double angle, int bgColor, double cx, double cy, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_hFlip(DrawAreaInternal *ptr);
CHARTDIR_DLLEXPORT void CDrawArea_vFlip(DrawAreaInternal *ptr);
CHARTDIR_DLLEXPORT void CDrawArea_clone(DrawAreaInternal *ptr, DrawAreaInternal *d, int x, int y, int align, int newWidth, int newHeight, int ft, double blur);

CHARTDIR_DLLEXPORT void CDrawArea_pixel(DrawAreaInternal *ptr, int x, int y, int c);
CHARTDIR_DLLEXPORT int CDrawArea_getPixel(DrawAreaInternal *ptr, int x, int y);

CHARTDIR_DLLEXPORT void CDrawArea_hline(DrawAreaInternal *ptr, int x1, int x2, int y, int c);
CHARTDIR_DLLEXPORT void CDrawArea_vline(DrawAreaInternal *ptr, int y1, int y2, int x, int c);
CHARTDIR_DLLEXPORT void CDrawArea_line(DrawAreaInternal *ptr, double x1, double y1, double x2, double y2, int c, int lineWidth);
CHARTDIR_DLLEXPORT void CDrawArea_arc(DrawAreaInternal *ptr, int cx, int cy, int rx, int ry, double a1, double a2, int c);

CHARTDIR_DLLEXPORT void CDrawArea_rect(DrawAreaInternal *ptr, int x1, int y1, int x2, int y2, int edgeColor, int fillColor, int raisedEffect);
CHARTDIR_DLLEXPORT void CDrawArea_polygon2(DrawAreaInternal *ptr, const int *xData, int xLen, const int *yData, int yLen, int edgeColor, int fillColor);
CHARTDIR_DLLEXPORT void CDrawArea_polygon(DrawAreaInternal *ptr, const double *xData, int xLen, const double *yData, int yLen, int edgeColor, int fillColor);
CHARTDIR_DLLEXPORT void CDrawArea_surface(DrawAreaInternal *ptr, double x1, double y1, double x2, double y2, int depthX, int depthY,
	int edgeColor, int fillColor);
CHARTDIR_DLLEXPORT void CDrawArea_sector(DrawAreaInternal *ptr, int cx, int cy, int rx, int ry, double a1, double a2, int edgeColor, int fillColor);
CHARTDIR_DLLEXPORT void CDrawArea_cylinder(DrawAreaInternal *ptr, int cx, int cy, int rx, int ry, double a1, double a2,
	int depthX, int depthY, int edgeColor, int fillColor);
CHARTDIR_DLLEXPORT void CDrawArea_circle(DrawAreaInternal *ptr, int cx, int cy, int rx, int ry, int edgeColor, int fillColor);

CHARTDIR_DLLEXPORT void CDrawArea_fill(DrawAreaInternal *ptr, int x, int y, int color);
CHARTDIR_DLLEXPORT void CDrawArea_fill2(DrawAreaInternal *ptr, int x, int y, int color, int borderColor);

CHARTDIR_DLLEXPORT void CDrawArea_text2(DrawAreaInternal *ptr, const char *str, const char *font, int fontIndex, double fontHeight, double fontWidth,
	double angle, bool vertical, int x, int y, int color, int align);
CHARTDIR_DLLEXPORT void CDrawArea_text2w(DrawAreaInternal *ptr, const wchar_t *str, const char *font, int fontIndex, double fontHeight, double fontWidth,
	double angle, bool vertical, int x, int y, int color, int align);
CHARTDIR_DLLEXPORT void CDrawArea_text(DrawAreaInternal *ptr, const char *str, const char *font, double fontSize,
	int x, int y, int color);
CHARTDIR_DLLEXPORT void CDrawArea_textw(DrawAreaInternal *ptr, const wchar_t *str, const char *font, double fontSize,
	int x, int y, int color);
CHARTDIR_DLLEXPORT TTFTextInternal* CDrawArea_text4(DrawAreaInternal *ptr, const wchar_t *text, const char *font, int fontIndex,
	double fontHeight, double fontWidth, double angle, bool vertical);
CHARTDIR_DLLEXPORT TTFTextInternal* CDrawArea_text4w(DrawAreaInternal *ptr, const char *text, const char *font, int fontIndex,
	double fontHeight, double fontWidth, double angle, bool vertical);
CHARTDIR_DLLEXPORT TTFTextInternal* CDrawArea_text3(DrawAreaInternal *ptr, const char *str, const char *font, double fontSize);
CHARTDIR_DLLEXPORT TTFTextInternal* CDrawArea_text3w(DrawAreaInternal *ptr, const wchar_t *str, const char *font, double fontSize);
CHARTDIR_DLLEXPORT void CDrawArea_close(DrawAreaInternal *ptr, TTFTextInternal *text);

CHARTDIR_DLLEXPORT void CDrawArea_merge(DrawAreaInternal *ptr, DrawAreaInternal *d, int x, int y, int align, int transparency);
CHARTDIR_DLLEXPORT void CDrawArea_tile(DrawAreaInternal *ptr, DrawAreaInternal *d, int transparency);

CHARTDIR_DLLEXPORT void CDrawArea_setSearchPath(DrawAreaInternal *ptr, const char *path);
CHARTDIR_DLLEXPORT bool CDrawArea_loadGIF(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_loadPNG(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_loadJPG(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_loadWMP(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_load(DrawAreaInternal *ptr, const char *filename);

CHARTDIR_DLLEXPORT void CDrawArea_rAffineTransform(DrawAreaInternal *ptr, double a, double b, double c, double d, double e, double f, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_affineTransform(DrawAreaInternal *ptr, double a, double b, double c, double d, double e, double f, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_sphereTransform(DrawAreaInternal *ptr, int xDiameter, int yDiameter, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_hCylinderTransform(DrawAreaInternal *ptr, int yDiameter, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_vCylinderTransform(DrawAreaInternal *ptr, int xDiameter, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_vTriangleTransform(DrawAreaInternal *ptr, int tHeight, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_hTriangleTransform(DrawAreaInternal *ptr, int tWidth, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_shearTransform(DrawAreaInternal *ptr, double xShear, double yShear, int bgColor, int ft, double blur);
CHARTDIR_DLLEXPORT void CDrawArea_waveTransform(DrawAreaInternal *ptr, int period, double amplitude, double direction, double startAngle, bool longitudinal, int bgColor, int ft, double blur);

CHARTDIR_DLLEXPORT bool CDrawArea_out(DrawAreaInternal *ptr, const char *filename);

CHARTDIR_DLLEXPORT bool CDrawArea_outGIF(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_outPNG(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_outJPG(DrawAreaInternal *ptr, const char *filename, int quality);
CHARTDIR_DLLEXPORT bool CDrawArea_outWMP(DrawAreaInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CDrawArea_outBMP(DrawAreaInternal *ptr, const char *filename);

CHARTDIR_DLLEXPORT bool CDrawArea_outGIF2(DrawAreaInternal *ptr, const char **data, int *len);
CHARTDIR_DLLEXPORT bool CDrawArea_outPNG2(DrawAreaInternal *ptr, const char **data, int *len);
CHARTDIR_DLLEXPORT bool CDrawArea_outJPG2(DrawAreaInternal *ptr, const char **data, int *len, int quality);
CHARTDIR_DLLEXPORT bool CDrawArea_outWMP2(DrawAreaInternal *ptr, const char **data, int *len);
CHARTDIR_DLLEXPORT bool CDrawArea_outBMP2(DrawAreaInternal *ptr, const char **data, int *len);

CHARTDIR_DLLEXPORT void CDrawArea_setPaletteMode(DrawAreaInternal *ptr, int p);
CHARTDIR_DLLEXPORT void CDrawArea_setDitherMethod(DrawAreaInternal *ptr, int m);
CHARTDIR_DLLEXPORT void CDrawArea_setTransparentColor(DrawAreaInternal *ptr, int c);
CHARTDIR_DLLEXPORT void CDrawArea_setAntiAliasText(DrawAreaInternal *ptr, int a);
CHARTDIR_DLLEXPORT void CDrawArea_setAntiAlias(DrawAreaInternal *ptr, bool shapeAntiAlias, int textAntiAlias);
CHARTDIR_DLLEXPORT void CDrawArea_setInterlace(DrawAreaInternal *ptr, bool i);

CHARTDIR_DLLEXPORT void CDrawArea_setColorTable(DrawAreaInternal *ptr, const int *colors, int noOfColors, int offset);
CHARTDIR_DLLEXPORT int CDrawArea_getARGBColor(DrawAreaInternal *ptr, int c);
CHARTDIR_DLLEXPORT int CDrawArea_halfColor(DrawAreaInternal *ptr, int c);

CHARTDIR_DLLEXPORT int CDrawArea_dashLineColor(DrawAreaInternal *ptr, int color, int dashPattern);
CHARTDIR_DLLEXPORT int CDrawArea_patternColor(DrawAreaInternal *ptr, const int *cData, int cLen, int h, int startX, int startY);
CHARTDIR_DLLEXPORT int CDrawArea_patternColor2(DrawAreaInternal *ptr, const char *filename, int startX, int startY);
CHARTDIR_DLLEXPORT int CDrawArea_gradientColor2a(DrawAreaInternal *ptr, const int *c, double angle, double scale, int startX, int startY);
CHARTDIR_DLLEXPORT int CDrawArea_gradientColor2(DrawAreaInternal *ptr, const int *cData, int cLen, double angle, double scale, int startX, int startY);
CHARTDIR_DLLEXPORT int CDrawArea_gradientColor(DrawAreaInternal *ptr, int startX, int startY, int endX, int endY, int startColor, int endColor);

CHARTDIR_DLLEXPORT void CDrawArea_setDefaultFonts(DrawAreaInternal *ptr, const char *normal, const char *bold, const char *italic, const char *boldItalic);
CHARTDIR_DLLEXPORT void CDrawArea_setFontTable(DrawAreaInternal *ptr, int index, const char *font);


////////////////////////////////////////////////////////////////////////////////////////
//	drawobj.h
////////////////////////////////////////////////////////////////////////////////////////
class DrawObjInternal;
CHARTDIR_DLLEXPORT void CDrawObj_paint(DrawObjInternal *ptr, DrawAreaInternal *d);


class BoxInternal;
CHARTDIR_DLLEXPORT DrawObjInternal *Box2DrawObj(BoxInternal *ptr);
CHARTDIR_DLLEXPORT void CBox_setPos(BoxInternal *ptr, int x, int y);
CHARTDIR_DLLEXPORT void CBox_setSize(BoxInternal *ptr, int w, int h);
CHARTDIR_DLLEXPORT void CBox_setBackground(BoxInternal *ptr, int color, int edgeColor, int raisedEffect);

CHARTDIR_DLLEXPORT int CBox_getWidth(BoxInternal *ptr);
CHARTDIR_DLLEXPORT int CBox_getHeight(BoxInternal *ptr);
CHARTDIR_DLLEXPORT const char *CBox_getImageCoor(BoxInternal *ptr, int offsetX, int offsetY);


class TextBoxInternal;
CHARTDIR_DLLEXPORT BoxInternal *TextBox2Box(TextBoxInternal *ptr);

CHARTDIR_DLLEXPORT void CTextBox_setText(TextBoxInternal *ptr, const char *text);
CHARTDIR_DLLEXPORT void CTextBox_setAlignment(TextBoxInternal *ptr, int a);
CHARTDIR_DLLEXPORT void CTextBox_setFontStyle(TextBoxInternal *ptr, const char *font, int fontIndex);
CHARTDIR_DLLEXPORT void CTextBox_setFontSize(TextBoxInternal *ptr, double fontHeight, double fontWidth);
CHARTDIR_DLLEXPORT void CTextBox_setFontAngle(TextBoxInternal *ptr, double angle, bool vertical);
CHARTDIR_DLLEXPORT void CTextBox_setFontColor(TextBoxInternal *ptr, int color);
CHARTDIR_DLLEXPORT void CTextBox_setMargin2(TextBoxInternal *ptr, int leftMargin, int rightMargin, int topMargin, int bottomMargin);
CHARTDIR_DLLEXPORT void CTextBox_setMargin(TextBoxInternal *ptr, int m);


class LineInternal;
CHARTDIR_DLLEXPORT DrawObjInternal *Line2DrawObj(LineInternal *ptr);

CHARTDIR_DLLEXPORT void CLine_setPos(LineInternal *ptr, int x1, int y1, int x2, int y2);
CHARTDIR_DLLEXPORT void CLine_setColor(LineInternal *ptr, int c);
CHARTDIR_DLLEXPORT void CLine_setWidth(LineInternal *ptr, int w);


////////////////////////////////////////////////////////////////////////////////////////
//	basechart.h
////////////////////////////////////////////////////////////////////////////////////////
class LegendBoxInternal;
CHARTDIR_DLLEXPORT TextBoxInternal *LegendBox2TextBox(LegendBoxInternal *ptr);

CHARTDIR_DLLEXPORT void CLegendBox_addKey(LegendBoxInternal *ptr, const char *text, int color, int lineWidth, DrawAreaInternal *symbol);
CHARTDIR_DLLEXPORT void CLegendBox_setKeySize(LegendBoxInternal *ptr, int width, int height, int gap);
CHARTDIR_DLLEXPORT void CLegendBox_setKeySpacing(LegendBoxInternal *ptr, int keySpacing, int lineSpacing);

CHARTDIR_DLLEXPORT const char *CLegendBox_getImageCoor(LegendBoxInternal *ptr, int dataItem, int offsetX, int offsetY);
CHARTDIR_DLLEXPORT const char *CLegendBox_getHTMLImageMap(LegendBoxInternal *ptr, const char *url, const char *queryFormat, const char *extraAttr, int offsetX, int offsetY);


class BaseChartInternal;
CHARTDIR_DLLEXPORT void CBaseChart_destroy(BaseChartInternal *ptr);

//////////////////////////////////////////////////////////////////////////////////////
//	set overall chart
//////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT void CBaseChart_setSize(BaseChartInternal *ptr, int width, int height);
CHARTDIR_DLLEXPORT void CBaseChart_setBackground(BaseChartInternal *ptr, int bgColor, int edgeColor, int raisedEffect);
CHARTDIR_DLLEXPORT void CBaseChart_setBorder(BaseChartInternal *ptr, int color);	//deprecated - use setBackground
CHARTDIR_DLLEXPORT void CBaseChart_setWallpaper(BaseChartInternal *ptr, const char *img);
CHARTDIR_DLLEXPORT void CBaseChart_setBgImage(BaseChartInternal *ptr, const char *img, int align);
CHARTDIR_DLLEXPORT void CBaseChart_setTransparentColor(BaseChartInternal *ptr, int c);
CHARTDIR_DLLEXPORT void CBaseChart_setAntiAlias(BaseChartInternal *ptr, bool antiAliasShape, int antiAliasText);
CHARTDIR_DLLEXPORT void CBaseChart_setSearchPath(BaseChartInternal *ptr, const char *path);

CHARTDIR_DLLEXPORT TextBoxInternal *CBaseChart_addTitle2(BaseChartInternal *ptr, int alignment, const char *text, const char *font, double fontSize, int fontColor, int bgColor, int edgeColor);
CHARTDIR_DLLEXPORT TextBoxInternal *CBaseChart_addTitle(BaseChartInternal *ptr, const char *text, const char *font, double fontSize, int fontColor, int bgColor, int edgeColor);

CHARTDIR_DLLEXPORT LegendBoxInternal *CBaseChart_addLegend(BaseChartInternal *ptr, int x, int y, bool vertical, const char *font, double fontHeight);
CHARTDIR_DLLEXPORT LegendBoxInternal *CBaseChart_getLegend(BaseChartInternal *ptr);

//////////////////////////////////////////////////////////////////////////////////////
//	drawing primitives
//////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT DrawAreaInternal *CBaseChart_getDrawArea(BaseChartInternal *ptr);
CHARTDIR_DLLEXPORT DrawObjInternal *CBaseChart_addDrawObj(BaseChartInternal *ptr, DrawObjInternal *obj);
CHARTDIR_DLLEXPORT TextBoxInternal *CBaseChart_addText(BaseChartInternal *ptr, int x, int y, const char *text, const char *font, double fontSize, int fontColor, int alignment, double angle, bool vertical);
CHARTDIR_DLLEXPORT LineInternal *CBaseChart_addLine(BaseChartInternal *ptr, int x1, int y1, int x2, int y2, int color, int lineWidth);

//////////////////////////////////////////////////////////////////////////////////////
//	color management methods
//////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT void CBaseChart_setColors(BaseChartInternal *ptr, int paletteEntry, const int *colorsData, int colorsLen);
CHARTDIR_DLLEXPORT int CBaseChart_getColor(BaseChartInternal *ptr, int paletteEntry);
CHARTDIR_DLLEXPORT int CBaseChart_halfColor(BaseChartInternal *ptr, int c);

CHARTDIR_DLLEXPORT int CBaseChart_dashLineColor(BaseChartInternal *ptr, int color, int patternCode);
CHARTDIR_DLLEXPORT int CBaseChart_patternColor(BaseChartInternal *ptr, const int *cData, int cLen, int h, int startX, int startY);
CHARTDIR_DLLEXPORT int CBaseChart_patternColor2(BaseChartInternal *ptr, const char *filename, int startX, int startY);
CHARTDIR_DLLEXPORT int CBaseChart_gradientColor2a(BaseChartInternal *ptr, const int *c, double angle, double scale, int startX, int startY);
CHARTDIR_DLLEXPORT int CBaseChart_gradientColor2(BaseChartInternal *ptr, const int *cData, int cLen, double angle, double scale, int startX, int startY);
CHARTDIR_DLLEXPORT int CBaseChart_gradientColor(BaseChartInternal *ptr, int startX, int startY, int endX, int endY, int startColor, int endColor);

//////////////////////////////////////////////////////////////////////////////////////
//	locale support
//////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT void CBaseChart_setDefaultFonts(BaseChartInternal *ptr, const char *normal, const char *bold, const char *italic, const char *boldItalic);
CHARTDIR_DLLEXPORT void CBaseChart_setFontTable(BaseChartInternal *ptr, int index, const char *font);

CHARTDIR_DLLEXPORT void CBaseChart_setNumberFormat(BaseChartInternal *ptr, char thousandSeparator, char decimalPointChar, char signChar);
CHARTDIR_DLLEXPORT void CBaseChart_setMonthNames(BaseChartInternal *ptr, const char * const *namesData, int namesLen);
CHARTDIR_DLLEXPORT void CBaseChart_setWeekDayNames(BaseChartInternal *ptr, const char * const *namesData, int namesLen);
CHARTDIR_DLLEXPORT void CBaseChart_setAMPM(BaseChartInternal *ptr, const char *AM, const char *PM);

//////////////////////////////////////////////////////////////////////////////////////
//	chart creation methods
//////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT void CBaseChart_layout(BaseChartInternal *ptr);
CHARTDIR_DLLEXPORT bool CBaseChart_makeChart(BaseChartInternal *ptr, const char *filename);
CHARTDIR_DLLEXPORT bool CBaseChart_makeChart2(BaseChartInternal *ptr, int format, const char **data, int *len);
CHARTDIR_DLLEXPORT DrawAreaInternal *CBaseChart_makeChart3(BaseChartInternal *ptr);

//////////////////////////////////////////////////////////////////////////////////////
//	image map support
//////////////////////////////////////////////////////////////////////////////////////
CHARTDIR_DLLEXPORT const char *CBaseChart_getHTMLImageMap(BaseChartInternal *ptr, const char *url, const char *queryFormat, const char *extraAttr, int offsetX, int offsetY);


class MultiChartInternal;
CHARTDIR_DLLEXPORT BaseChartInternal *MultiChart2BaseChart(MultiChartInternal *ptr);
CHARTDIR_DLLEXPORT MultiChartInternal *CMultiChart_create(int width, int height, int bgColor, int edgeColor, int raisedEffect);
CHARTDIR_DLLEXPORT void CMultiChart_addChart(MultiChartInternal *ptr, int x, int y, BaseChartInternal *c);


////////////////////////////////////////////////////////////////////////////////////////
//	piechart.h
////////////////////////////////////////////////////////////////////////////////////////
class SectorInternal;
CHARTDIR_DLLEXPORT void CSector_setExplode(SectorInternal *ptr, int distance);
CHARTDIR_DLLEXPORT void CSector_setLabelFormat(SectorInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT TextBoxInternal *CSector_setLabelStyle(SectorInternal *ptr, const char *font, double fontSize, int fontColor);
CHARTDIR_DLLEXPORT void CSector_setLabelPos(SectorInternal *ptr, int pos, int joinLineColor);
CHARTDIR_DLLEXPORT void CSector_setLabelLayout(SectorInternal *ptr, int layoutMethod, int pos);
CHARTDIR_DLLEXPORT void CSector_setJoinLine(SectorInternal *ptr, int joinLineColor, int joinLineWidth);
CHARTDIR_DLLEXPORT void CSector_setColor(SectorInternal *ptr, int color, int edgeColor, int joinLineColor);
CHARTDIR_DLLEXPORT const char *CSector_getImageCoor(SectorInternal *ptr, int offsetX, int offsetY);
CHARTDIR_DLLEXPORT const char *CSector_getLabelCoor(SectorInternal *ptr, int offsetX, int offsetY);


class PieChartInternal;
CHARTDIR_DLLEXPORT BaseChartInternal *PieChart2BaseChart(PieChartInternal *ptr);
CHARTDIR_DLLEXPORT PieChartInternal *CPieChart_create(int width, int height, int bgColor, int edgeColor, int raisedEffect);

CHARTDIR_DLLEXPORT void CPieChart_setPieSize(PieChartInternal *ptr, int x, int y, int r);
CHARTDIR_DLLEXPORT void CPieChart_set3D(PieChartInternal *ptr, int depth, double angle, bool shadowMode);
CHARTDIR_DLLEXPORT void CPieChart_set3D2(PieChartInternal *ptr, const double *depthsData, int depthsLen, double angle, bool shadowMode);
CHARTDIR_DLLEXPORT void CPieChart_setStartAngle(PieChartInternal *ptr, double startAngle, bool clockWise);
CHARTDIR_DLLEXPORT void CPieChart_setExplode(PieChartInternal *ptr, int sectorNo, int distance);
CHARTDIR_DLLEXPORT void CPieChart_setExplodeGroup(PieChartInternal *ptr, int startSector, int endSector, int distance);

CHARTDIR_DLLEXPORT void CPieChart_setLabelFormat(PieChartInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT TextBoxInternal *CPieChart_setLabelStyle(PieChartInternal *ptr, const char *font, double fontSize, int fontColor);
CHARTDIR_DLLEXPORT void CPieChart_setLabelPos(PieChartInternal *ptr, int pos, int joinLineColor);
CHARTDIR_DLLEXPORT void CPieChart_setLabelLayout(PieChartInternal *ptr, int layoutMethod, int pos, int topBound, int bottomBound);
CHARTDIR_DLLEXPORT void CPieChart_setJoinLine(PieChartInternal *ptr, int joinLineColor, int joinLineWidth);
CHARTDIR_DLLEXPORT void CPieChart_setLineColor(PieChartInternal *ptr, int edgeColor, int joinLineColor);

CHARTDIR_DLLEXPORT void CPieChart_setData(PieChartInternal *ptr, const double *dataData, int dataLen, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT void CPieChart_addExtraField(PieChartInternal *ptr, const char * const * textsData, int textsLen);

CHARTDIR_DLLEXPORT SectorInternal *CPieChart_sector(PieChartInternal *ptr, int sectorNo);


////////////////////////////////////////////////////////////////////////////////////////
//	axis.h
////////////////////////////////////////////////////////////////////////////////////////
class MarkInternal;
class LineObjInternal;
CHARTDIR_DLLEXPORT TextBoxInternal *Mark2TextBox(MarkInternal *ptr);
CHARTDIR_DLLEXPORT void CMark_setValue(MarkInternal *ptr, double value);
CHARTDIR_DLLEXPORT void CMark_setMarkColor(MarkInternal *ptr, int lineColor, int textColor, int tickColor);
CHARTDIR_DLLEXPORT void CMark_setLineWidth(MarkInternal *ptr, int w);
CHARTDIR_DLLEXPORT void CMark_setDrawOnTop(MarkInternal *ptr, bool b);
CHARTDIR_DLLEXPORT LineObjInternal *CMark_getLine(MarkInternal *ptr);

//
//	XYChart Axis
//
class AxisInternal;

//
//	axis appearance
//
CHARTDIR_DLLEXPORT TextBoxInternal *CAxis_setLabelStyle(AxisInternal *ptr, const char *font, double fontSize, int fontColor, double fontAngle);
CHARTDIR_DLLEXPORT void CAxis_setLabelFormat(AxisInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT void CAxis_setLabelGap(AxisInternal *ptr, int d);

CHARTDIR_DLLEXPORT TextBoxInternal *CAxis_setTitle(AxisInternal *ptr, const char *text, const char *font, double fontSize, int fontColor);
CHARTDIR_DLLEXPORT void CAxis_setTitlePos(AxisInternal *ptr, int alignment, int titleGap);

CHARTDIR_DLLEXPORT void CAxis_setColors(AxisInternal *ptr, int axisColor, int labelColor, int titleColor, int tickColor);

CHARTDIR_DLLEXPORT void CAxis_setTickLength(AxisInternal *ptr, int majorTickLen);
CHARTDIR_DLLEXPORT void CAxis_setTickLength2(AxisInternal *ptr, int majorTickLen, int minorTickLen);
CHARTDIR_DLLEXPORT void CAxis_setTickWidth(AxisInternal *ptr, int majorTickWidth, int minorTickWidth);
CHARTDIR_DLLEXPORT void CAxis_setTickColor(AxisInternal *ptr, int majorTickColor, int minorTickColor);

//
//	axis position
//
CHARTDIR_DLLEXPORT void CAxis_setWidth(AxisInternal *ptr, int width);
CHARTDIR_DLLEXPORT void CAxis_setLength(AxisInternal *ptr, int length);
CHARTDIR_DLLEXPORT void CAxis_setPos(AxisInternal *ptr, int x, int y, int align);
CHARTDIR_DLLEXPORT void CAxis_setAngle(AxisInternal *ptr, double startAngle);
CHARTDIR_DLLEXPORT void CAxis_setMargin(AxisInternal *ptr, int topMargin, int bottomMargin);
CHARTDIR_DLLEXPORT void CAxis_setIndent(AxisInternal *ptr, bool indent);

//
//	axis scaling
//
CHARTDIR_DLLEXPORT void CAxis_setAutoScale(AxisInternal *ptr, double topExtension, double bottomExtension, double zeroAffinity);
CHARTDIR_DLLEXPORT void CAxis_setRounding(AxisInternal *ptr, bool roundMin, bool roundMax);
CHARTDIR_DLLEXPORT void CAxis_setTickDensity(AxisInternal *ptr, int majorTickSpacing, int minorTickSpacing);
CHARTDIR_DLLEXPORT void CAxis_setReverse(AxisInternal *ptr, bool b);

CHARTDIR_DLLEXPORT TextBoxInternal *CAxis_setLabels(AxisInternal *ptr, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT TextBoxInternal *CAxis_setLabels2(AxisInternal *ptr, const double *labelsData, int labelsLen, const char *formatString);

CHARTDIR_DLLEXPORT void CAxis_setLinearScale3(AxisInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT void CAxis_setLinearScale2(AxisInternal *ptr, double lowerLimit, double upperLimit, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT void CAxis_setLinearScale(AxisInternal *ptr, double lowerLimit, double upperLimit, double majorTickInc, double minorTickInc);

CHARTDIR_DLLEXPORT void CAxis_setLogScale3(AxisInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT void CAxis_setLogScale2(AxisInternal *ptr, double lowerLimit, double upperLimit, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT void CAxis_setLogScale(AxisInternal *ptr, double lowerLimit, double upperLimit, double majorTickInc, double minorTickInc);

CHARTDIR_DLLEXPORT void CAxis_setDateScale3(AxisInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT void CAxis_setDateScale2(AxisInternal *ptr, double lowerLimit, double upperLimit, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT void CAxis_setDateScale(AxisInternal *ptr, double lowerLimit, double upperLimit, double majorTickInc, double minorTickInc);

//
//	custom markings
//
CHARTDIR_DLLEXPORT void CAxis_addLabel(AxisInternal *ptr, double pos, const char *label);

CHARTDIR_DLLEXPORT MarkInternal *CAxis_addMark(AxisInternal *ptr, double value, int lineColor, const char *text, const char *font, double fontSize);
CHARTDIR_DLLEXPORT void CAxis_addZone(AxisInternal *ptr, double startValue, double endValue, int color);

//
//	axis querying
//
CHARTDIR_DLLEXPORT int CAxis_getCoor(AxisInternal *ptr, double v);
CHARTDIR_DLLEXPORT double CAxis_getMinValue(AxisInternal *ptr);
CHARTDIR_DLLEXPORT double CAxis_getMaxValue(AxisInternal *ptr);

CHARTDIR_DLLEXPORT void CAxis_getTicks(AxisInternal *ptr, const double **d, int *len);
CHARTDIR_DLLEXPORT const char *CAxis_getLabel(AxisInternal *ptr, double i);


typedef AxisInternal RadialAxisInternal;

//
//	AngularAxis for PolarChart
//
class AngularAxisInternal;

//
//	axis appearance
//
CHARTDIR_DLLEXPORT TextBoxInternal *CAngularAxis_setLabelStyle(AngularAxisInternal *ptr, const char *font, double fontSize, int fontColor, double fontAngle);
CHARTDIR_DLLEXPORT void CAngularAxis_setLabelGap(AngularAxisInternal *ptr, int d);

//
//	axis position
//
CHARTDIR_DLLEXPORT void CAngularAxis_setAngle(AngularAxisInternal *ptr, double axisAngle);
CHARTDIR_DLLEXPORT void CAngularAxis_setReverse(AngularAxisInternal *ptr, bool b);

//
//	axis scaling
//
CHARTDIR_DLLEXPORT TextBoxInternal *CAngularAxis_setLabels(AngularAxisInternal *ptr, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT TextBoxInternal *CAngularAxis_setLabels2(AngularAxisInternal *ptr, const double * labelsData, int labelsLen, const char *formatString);
CHARTDIR_DLLEXPORT void CAngularAxis_addLabel(AngularAxisInternal *ptr, double pos, const char *label);

CHARTDIR_DLLEXPORT void CAngularAxis_setLinearScale2(AngularAxisInternal *ptr, double lowerLimit, double upperLimit, const char * const * labelsData, int labelsLen);
CHARTDIR_DLLEXPORT void CAngularAxis_setLinearScale(AngularAxisInternal *ptr, double lowerLimit, double upperLimit, double majorTickInc, double minorTickInc);


////////////////////////////////////////////////////////////////////////////////////////
//	layer.h
////////////////////////////////////////////////////////////////////////////////////////
class DataSetInternal;
CHARTDIR_DLLEXPORT void CDataSet_setData(DataSetInternal *ptr, const double *dataData, int dataLen);
CHARTDIR_DLLEXPORT void CDataSet_setDataName(DataSetInternal *ptr, const char *name);
CHARTDIR_DLLEXPORT void CDataSet_setDataColor(DataSetInternal *ptr, int dataColor, int edgeColor, int shadowColor, int shadowEdgeColor);

CHARTDIR_DLLEXPORT void CDataSet_setDataSymbol2(DataSetInternal *ptr, const char *image);
CHARTDIR_DLLEXPORT void CDataSet_setDataSymbol3(DataSetInternal *ptr, DrawAreaInternal *image);
CHARTDIR_DLLEXPORT void CDataSet_setDataSymbol(DataSetInternal *ptr, int symbol, int size, int fillColor, int edgeColor, int lineWidth);

CHARTDIR_DLLEXPORT void CDataSet_setLineWidth(DataSetInternal *ptr, int w);

CHARTDIR_DLLEXPORT void CDataSet_setDataLabelFormat(DataSetInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT TextBoxInternal *CDataSet_setDataLabelStyle(DataSetInternal *ptr, const char *font, double fontSize, int fontColor, double fontAngle);

CHARTDIR_DLLEXPORT void CDataSet_setUseYAxis2(DataSetInternal *ptr, bool b);


class LayerInternal;
CHARTDIR_DLLEXPORT void CLayer_setSize(LayerInternal *ptr, int x, int y, int w, int h, bool swapXY);
CHARTDIR_DLLEXPORT void CLayer_set3D(LayerInternal *ptr, int d, int zGap);
CHARTDIR_DLLEXPORT void CLayer_setLineWidth(LayerInternal *ptr, int w);
CHARTDIR_DLLEXPORT void CLayer_setBorderColor(LayerInternal *ptr, int color, int raisedEffect);
CHARTDIR_DLLEXPORT void CLayer_setLegend(LayerInternal *ptr, int m);

CHARTDIR_DLLEXPORT void CLayer_setDataCombineMethod(LayerInternal *ptr, int m);
CHARTDIR_DLLEXPORT DataSetInternal *CLayer_addDataSet(LayerInternal *ptr, const double *dataData, int dataLen, int color, const char *name);
CHARTDIR_DLLEXPORT void CLayer_addDataGroup(LayerInternal *ptr, const char *name);
CHARTDIR_DLLEXPORT void CLayer_addExtraField(LayerInternal *ptr, const char * const * textsData, int textsLen);
CHARTDIR_DLLEXPORT DataSetInternal *CLayer_getDataSet(LayerInternal *ptr, int dataSet);
CHARTDIR_DLLEXPORT void CLayer_setUseYAxis2(LayerInternal *ptr, bool b);

CHARTDIR_DLLEXPORT void CLayer_setXData(LayerInternal *ptr, const double *xDataData, int xDataLen);
CHARTDIR_DLLEXPORT void CLayer_setXData2(LayerInternal *ptr, double minValue, double maxValue);

CHARTDIR_DLLEXPORT int CLayer_getXCoor(LayerInternal *ptr, double v);
CHARTDIR_DLLEXPORT int CLayer_getYCoor(LayerInternal *ptr, double v, bool yAxis);
CHARTDIR_DLLEXPORT int CLayer_xZoneColor(LayerInternal *ptr, double threshold, int belowColor, int aboveColor);
CHARTDIR_DLLEXPORT int CLayer_yZoneColor(LayerInternal *ptr, double threshold, int belowColor, int aboveColor, bool mainAxis);

CHARTDIR_DLLEXPORT const char *CLayer_getImageCoor(LayerInternal *ptr, int dataSet, int dataItem, int offsetX, int offsetY);
CHARTDIR_DLLEXPORT const char *CLayer_getImageCoor2(LayerInternal *ptr, int dataItem, int offsetX, int offsetY);
CHARTDIR_DLLEXPORT const char *CLayer_getHTMLImageMap(LayerInternal *ptr, const char *url, const char *queryFormat, const char *extraAttr, int offsetX, int offsetY);

CHARTDIR_DLLEXPORT void CLayer_setDataLabelFormat(LayerInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT TextBoxInternal *CLayer_setDataLabelStyle(LayerInternal *ptr, const char *font, double fontSize, int fontColor, double fontAngle);

CHARTDIR_DLLEXPORT void CLayer_setAggregateLabelFormat(LayerInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT TextBoxInternal *CLayer_setAggregateLabelStyle(LayerInternal *ptr, const char *font, double fontSize, int fontColor, double fontAngle);

CHARTDIR_DLLEXPORT TextBoxInternal *CLayer_addCustomDataLabel(LayerInternal *ptr, int dataSet, int dataItem, const char *label, const char *font, double fontSize, int fontColor, double fontAngle);
CHARTDIR_DLLEXPORT TextBoxInternal *CLayer_addCustomAggregateLabel(LayerInternal *ptr, int dataItem, const char *label, const char *font, double fontSize, int fontColor, double fontAngle);


////////////////////////////////////////////////////////////////////////////////////////
//	barlayer.h
////////////////////////////////////////////////////////////////////////////////////////
class BarLayerInternal;
CHARTDIR_DLLEXPORT LayerInternal *BarLayer2Layer(BarLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CBarLayer_setBarGap(BarLayerInternal *ptr, double barGap, double subBarGap);
CHARTDIR_DLLEXPORT void CBarLayer_setBarWidth(BarLayerInternal *ptr, int barWidth, int subBarWidth);
CHARTDIR_DLLEXPORT void CBarLayer_setMinLabelSize(BarLayerInternal *ptr, int s);


////////////////////////////////////////////////////////////////////////////////////////
//	linelayer.h
////////////////////////////////////////////////////////////////////////////////////////
class InterLineLayerInternal;
CHARTDIR_DLLEXPORT LayerInternal *InterLineLayer2Layer(InterLineLayerInternal *ptr);

class LineLayerInternal;
CHARTDIR_DLLEXPORT LayerInternal *LineLayer2Layer(LineLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CLineLayer_setSymbolScale(LineLayerInternal *ptr, const double *zDataXData, int zDataXLen, int scaleTypeX, const double *zDataYData, int zDataYLen, int scaleTypeY);
CHARTDIR_DLLEXPORT void CLineLayer_setGapColor(LineLayerInternal *ptr, int lineColor, int lineWidth);
CHARTDIR_DLLEXPORT void CLineLayer_setImageMapWidth(LineLayerInternal *ptr, int width);
CHARTDIR_DLLEXPORT LineObjInternal *CLineLayer_getLine(LineLayerInternal *ptr, int dataSet);

class SplineLayerInternal;
CHARTDIR_DLLEXPORT LineLayerInternal *SplineLayer2LineLayer(SplineLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CSplineLayer_setTension(SplineLayerInternal *ptr, double tension);

class StepLineLayerInternal;
CHARTDIR_DLLEXPORT LineLayerInternal *StepLineLayer2LineLayer(StepLineLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CStepLineLayer_setAlignment(StepLineLayerInternal *ptr, int a);

typedef LineLayerInternal ScatterLayerInternal;


////////////////////////////////////////////////////////////////////////////////////////
//	arealayer.h
////////////////////////////////////////////////////////////////////////////////////////
class AreaLayerInternal;
CHARTDIR_DLLEXPORT LayerInternal *AreaLayer2Layer(AreaLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CAreaLayer_setMinLabelSize(AreaLayerInternal *ptr, int s);


////////////////////////////////////////////////////////////////////////////////////////
//	hloclayer.h
////////////////////////////////////////////////////////////////////////////////////////
class HLOCLayerInternal;
CHARTDIR_DLLEXPORT LayerInternal *HLOCLayer2Layer(HLOCLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CHLOCLayer_setDataGap(HLOCLayerInternal *ptr, double gap);

typedef HLOCLayerInternal CandleStickLayerInternal;
typedef HLOCLayerInternal BoxWhiskerLayerInternal;


////////////////////////////////////////////////////////////////////////////////////////
//	trendlayer.h
////////////////////////////////////////////////////////////////////////////////////////
class TrendLayerInternal;
CHARTDIR_DLLEXPORT LayerInternal *TrendLayer2Layer(TrendLayerInternal *ptr);

CHARTDIR_DLLEXPORT void CTrendLayer_addConfidenceBand(TrendLayerInternal *ptr, double confidence, int upperFillColor, int upperEdgeColor, int upperLineWidth, int lowerFillColor, int lowerEdgeColor, int lowerLineWidth);
CHARTDIR_DLLEXPORT void CTrendLayer_addPredictionBand(TrendLayerInternal *ptr, double confidence, int upperFillColor, int upperEdgeColor, int upperLineWidth, int lowerFillColor, int lowerEdgeColor, int lowerLineWidth);

CHARTDIR_DLLEXPORT double CTrendLayer_getSlope(TrendLayerInternal *ptr);
CHARTDIR_DLLEXPORT double CTrendLayer_getIntercept(TrendLayerInternal *ptr);
CHARTDIR_DLLEXPORT double CTrendLayer_getCorrelation(TrendLayerInternal *ptr);
CHARTDIR_DLLEXPORT double CTrendLayer_getStdError(TrendLayerInternal *ptr);

CHARTDIR_DLLEXPORT void CTrendLayer_setImageMapWidth(TrendLayerInternal *ptr, int width);
CHARTDIR_DLLEXPORT LineObjInternal *CTrendLayer_getLine(TrendLayerInternal *ptr);


////////////////////////////////////////////////////////////////////////////////////////
//	xychart.h
////////////////////////////////////////////////////////////////////////////////////////
class PlotAreaInternal;
CHARTDIR_DLLEXPORT void CPlotArea_setBackground(PlotAreaInternal *ptr, int color, int altBgColor, int edgeColor);
CHARTDIR_DLLEXPORT void CPlotArea_setBackground2(PlotAreaInternal *ptr, const char *img, int align);
CHARTDIR_DLLEXPORT void CPlotArea_setGridColor(PlotAreaInternal *ptr, int hGridColor, int vGridColor, int minorHGridColor, int minorVGridColor);
CHARTDIR_DLLEXPORT void CPlotArea_setGridWidth(PlotAreaInternal *ptr, int hGridWidth, int vGridWidth, int minorHGridWidth, int minorVGridWidth);


class XYChartInternal;
CHARTDIR_DLLEXPORT BaseChartInternal *XYChart2BaseChart(XYChartInternal *ptr);
CHARTDIR_DLLEXPORT XYChartInternal *CXYChart_create(int width, int height, int bgColor,	int edgeColor, int raisedEffect);

CHARTDIR_DLLEXPORT AxisInternal *CXYChart_yAxis(XYChartInternal *ptr);
CHARTDIR_DLLEXPORT AxisInternal *CXYChart_yAxis2(XYChartInternal *ptr);
CHARTDIR_DLLEXPORT void CXYChart_syncYAxis(XYChartInternal *ptr, double slope, double intercept);
CHARTDIR_DLLEXPORT void CXYChart_setYAxisOnRight(XYChartInternal *ptr, bool b);
CHARTDIR_DLLEXPORT AxisInternal *CXYChart_xAxis(XYChartInternal *ptr);
CHARTDIR_DLLEXPORT AxisInternal *CXYChart_xAxis2(XYChartInternal *ptr);
CHARTDIR_DLLEXPORT void CXYChart_setXAxisOnTop(XYChartInternal *ptr, bool b);
CHARTDIR_DLLEXPORT void CXYChart_swapXY(XYChartInternal *ptr, bool b);

CHARTDIR_DLLEXPORT PlotAreaInternal *CXYChart_setPlotArea(XYChartInternal *ptr, int x, int y, int width, int height, int bgColor, int altBgColor, int edgeColor, int hGridColor, int vGridColor);
CHARTDIR_DLLEXPORT void CXYChart_setClipping(XYChartInternal *ptr, int margin);

CHARTDIR_DLLEXPORT BarLayerInternal *CXYChart_addBarLayer(XYChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name, int depth);
CHARTDIR_DLLEXPORT BarLayerInternal *CXYChart_addBarLayer3(XYChartInternal *ptr, const double *dataData, int dataLen, const int *colorsData, int colorsLen, const char * const *namesData, int namesLen, int depth);
CHARTDIR_DLLEXPORT BarLayerInternal *CXYChart_addBarLayer2(XYChartInternal *ptr, int dataCombineMethod, int depth);

CHARTDIR_DLLEXPORT LineLayerInternal *CXYChart_addLineLayer(XYChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name, int depth);
CHARTDIR_DLLEXPORT LineLayerInternal *CXYChart_addLineLayer2(XYChartInternal *ptr, int dataCombineMethod, int depth);

CHARTDIR_DLLEXPORT AreaLayerInternal *CXYChart_addAreaLayer(XYChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name, int depth);
CHARTDIR_DLLEXPORT AreaLayerInternal *CXYChart_addAreaLayer2(XYChartInternal *ptr, int dataCombineMethod, int depth);

CHARTDIR_DLLEXPORT HLOCLayerInternal *CXYChart_addHLOCLayer2(XYChartInternal *ptr);
CHARTDIR_DLLEXPORT HLOCLayerInternal *CXYChart_addHLOCLayer(XYChartInternal *ptr, const double *highDataData, int highDataLen,
	const double *lowDataData, int lowDataLen, const double *openDataData, int openDataLen,
	const double *closeDataData, int closeDataLen, int color);

CHARTDIR_DLLEXPORT CandleStickLayerInternal *CXYChart_addCandleStickLayer(XYChartInternal *ptr, const double *highDataData, int highDataLen,
	const double *lowDataData, int lowDataLen, const double *openDataData, int openDataLen,
	const double *closeDataData, int closeDataLen, int riseColor, int fallColor, int edgeColor);

CHARTDIR_DLLEXPORT BoxWhiskerLayerInternal *CXYChart_addBoxWhiskerLayer(XYChartInternal *ptr, const double *boxTopData, int boxTopLen,
	const double *boxBottomData, int boxBottomLen, const double *maxDataData, int maxDataLen,
	const double *minDataData, int minDataLen, const double *midDataData, int midDataLen,
	int fillColor, int whiskerColor, int edgeColor);

CHARTDIR_DLLEXPORT ScatterLayerInternal *CXYChart_addScatterLayer(XYChartInternal *ptr, const double *xDataData, int xDataLen, const double *yDataData, int yDataLen, const char *name,
	int symbol, int symbolSize, int fillColor, int edgeColor);

CHARTDIR_DLLEXPORT TrendLayerInternal *CXYChart_addTrendLayer(XYChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name, int depth);
CHARTDIR_DLLEXPORT TrendLayerInternal *CXYChart_addTrendLayer2(XYChartInternal *ptr, const double *xDataData, int xDataLen, const double *yDataData, int yDataLen, int color, const char *name, int depth);

CHARTDIR_DLLEXPORT SplineLayerInternal *CXYChart_addSplineLayer(XYChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name);
CHARTDIR_DLLEXPORT StepLineLayerInternal *CXYChart_addStepLineLayer(XYChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name);

CHARTDIR_DLLEXPORT InterLineLayerInternal *CXYChart_addInterLineLayer(XYChartInternal *ptr, LineObjInternal *line1, LineObjInternal *line2, int color12, int color21);


////////////////////////////////////////////////////////////////////////////////////////
//	polarchart.h
////////////////////////////////////////////////////////////////////////////////////////
class PolarLayerInternal;
CHARTDIR_DLLEXPORT void CPolarLayer_setData(PolarLayerInternal *ptr, const double *dataData, int dataLen, int color, const char *name);
CHARTDIR_DLLEXPORT void CPolarLayer_setAngles(PolarLayerInternal *ptr, const double *anglesData, int anglesLen);
CHARTDIR_DLLEXPORT void CPolarLayer_setBorderColor(PolarLayerInternal *ptr, int edgeColor);
CHARTDIR_DLLEXPORT void CPolarLayer_setLineWidth(PolarLayerInternal *ptr, int w);
CHARTDIR_DLLEXPORT void CPolarLayer_setDataSymbol2(PolarLayerInternal *ptr, const char *image);
CHARTDIR_DLLEXPORT void CPolarLayer_setDataSymbol3(PolarLayerInternal *ptr, DrawAreaInternal *image);
CHARTDIR_DLLEXPORT void CPolarLayer_setDataSymbol(PolarLayerInternal *ptr, int symbol, int size, int fillColor, int edgeColor, int lineWidth);
CHARTDIR_DLLEXPORT void CPolarLayer_setSymbolScale(PolarLayerInternal *ptr, const double *zDataData, int zDataLen, int scaleType);
CHARTDIR_DLLEXPORT void CPolarLayer_setImageMapWidth(PolarLayerInternal *ptr, int width);
CHARTDIR_DLLEXPORT const char *CPolarLayer_getImageCoor(PolarLayerInternal *ptr, int dataItem, int offsetX, int offsetY);
CHARTDIR_DLLEXPORT const char *CPolarLayer_getHTMLImageMap(PolarLayerInternal *ptr, const char *url, const char *queryFormat, const char *extraAttr, int offsetX, int offsetY);
CHARTDIR_DLLEXPORT void CPolarLayer_setDataLabelFormat(PolarLayerInternal *ptr, const char *formatString);
CHARTDIR_DLLEXPORT TextBoxInternal *CPolarLayer_setDataLabelStyle(PolarLayerInternal *ptr, const char *font, double fontSize, int fontColor, double fontAngle);
CHARTDIR_DLLEXPORT TextBoxInternal *CPolarLayer_addCustomDataLabel(PolarLayerInternal *ptr, int i, const char *label, const char *font, double fontSize, int fontColor, double fontAngle);

class PolarAreaLayerInternal;
CHARTDIR_DLLEXPORT PolarLayerInternal *PolarAreaLayer2PolarLayer(PolarAreaLayerInternal *ptr);

class PolarLineLayerInternal;
CHARTDIR_DLLEXPORT PolarLayerInternal *PolarLineLayer2PolarLayer(PolarLineLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CPolarLineLayer_setCloseLoop(PolarLineLayerInternal *ptr, bool b);
CHARTDIR_DLLEXPORT void CPolarLineLayer_setGapColor(PolarLineLayerInternal *ptr, int lineColor, int lineWidth);

class PolarSplineLineLayerInternal;
CHARTDIR_DLLEXPORT PolarLineLayerInternal *PolarSplineLineLayer2PolarLineLayer(PolarSplineLineLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CPolarSplineLineLayer_setTension(PolarSplineLineLayerInternal *ptr, double tension);

class PolarSplineAreaLayerInternal;
CHARTDIR_DLLEXPORT PolarAreaLayerInternal *PolarSplineAreaLayer2PolarAreaLayer(PolarSplineAreaLayerInternal *ptr);
CHARTDIR_DLLEXPORT void CPolarSplineAreaLayer_setTension(PolarSplineAreaLayerInternal *ptr, double tension);

class PolarChartInternal;
CHARTDIR_DLLEXPORT BaseChartInternal *PolarChart2BaseChart(PolarChartInternal *ptr);
CHARTDIR_DLLEXPORT PolarChartInternal *CPolarChart_create(int width, int height, int bgColor, int edgeColor, int raisedEffect);

CHARTDIR_DLLEXPORT void CPolarChart_setPlotArea(PolarChartInternal *ptr, int x, int y, int r, int bgColor, int edgeColor, int edgeWidth);
CHARTDIR_DLLEXPORT void CPolarChart_setGridColor(PolarChartInternal *ptr, int rGridColor, int rGridWidth, int aGridColor, int aGridWidth);
CHARTDIR_DLLEXPORT void CPolarChart_setGridStyle(PolarChartInternal *ptr, bool polygonGrid, bool gridOnTop);
CHARTDIR_DLLEXPORT void CPolarChart_setStartAngle(PolarChartInternal *ptr, double startAngle, bool clockwise);

CHARTDIR_DLLEXPORT AngularAxisInternal *CPolarChart_angularAxis(PolarChartInternal *ptr);
CHARTDIR_DLLEXPORT RadialAxisInternal *CPolarChart_radialAxis(PolarChartInternal *ptr);

CHARTDIR_DLLEXPORT int CPolarChart_getXCoor(PolarChartInternal *ptr, double r, double a);
CHARTDIR_DLLEXPORT int CPolarChart_getYCoor(PolarChartInternal *ptr, double r, double a);

CHARTDIR_DLLEXPORT void CPolarChart_addExtraField(PolarChartInternal *ptr, const char * const * textsData, int textsLen);

CHARTDIR_DLLEXPORT PolarAreaLayerInternal *CPolarChart_addAreaLayer(PolarChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name);
CHARTDIR_DLLEXPORT PolarLineLayerInternal *CPolarChart_addLineLayer(PolarChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name);
CHARTDIR_DLLEXPORT PolarSplineLineLayerInternal *CPolarChart_addSplineLineLayer(PolarChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name);
CHARTDIR_DLLEXPORT PolarSplineAreaLayerInternal *CPolarChart_addSplineAreaLayer(PolarChartInternal *ptr, const double *dataData, int dataLen, int color, const char *name);


////////////////////////////////////////////////////////////////////////////////////////
//	datafilter.h
////////////////////////////////////////////////////////////////////////////////////////
class ArrayMathInternal;
CHARTDIR_DLLEXPORT ArrayMathInternal *CArrayMath_create(const double *aData, int aLen);
CHARTDIR_DLLEXPORT void CArrayMath_destroy(ArrayMathInternal *ptr);

CHARTDIR_DLLEXPORT void CArrayMath_add(ArrayMathInternal *ptr, const double *bData, int bLen);
CHARTDIR_DLLEXPORT void CArrayMath_add2(ArrayMathInternal *ptr, double b);
CHARTDIR_DLLEXPORT void CArrayMath_sub(ArrayMathInternal *ptr, const double *bData, int bLen);
CHARTDIR_DLLEXPORT void CArrayMath_sub2(ArrayMathInternal *ptr, double b);
CHARTDIR_DLLEXPORT void CArrayMath_mul(ArrayMathInternal *ptr, const double *bData, int bLen);
CHARTDIR_DLLEXPORT void CArrayMath_mul2(ArrayMathInternal *ptr, double b);
CHARTDIR_DLLEXPORT void CArrayMath_div(ArrayMathInternal *ptr, const double *bData, int bLen);
CHARTDIR_DLLEXPORT void CArrayMath_div2(ArrayMathInternal *ptr, double b);
CHARTDIR_DLLEXPORT void CArrayMath_delta(ArrayMathInternal *ptr, int offset);
CHARTDIR_DLLEXPORT void CArrayMath_abs(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT void CArrayMath_acc(ArrayMathInternal *ptr);

CHARTDIR_DLLEXPORT void CArrayMath_selectGTZ(ArrayMathInternal *ptr, const double *bData, int bLen, double fillValue);
CHARTDIR_DLLEXPORT void CArrayMath_selectGEZ(ArrayMathInternal *ptr, const double *bData, int bLen, double fillValue);
CHARTDIR_DLLEXPORT void CArrayMath_selectLTZ(ArrayMathInternal *ptr, const double *bData, int bLen, double fillValue);
CHARTDIR_DLLEXPORT void CArrayMath_selectLEZ(ArrayMathInternal *ptr, const double *bData, int bLen, double fillValue);
CHARTDIR_DLLEXPORT void CArrayMath_selectEQZ(ArrayMathInternal *ptr, const double *bData, int bLen, double fillValue);
CHARTDIR_DLLEXPORT void CArrayMath_selectNEZ(ArrayMathInternal *ptr, const double *bData, int bLen, double fillValue);

CHARTDIR_DLLEXPORT void CArrayMath_selectStartOfHour(ArrayMathInternal *ptr, int majorTickStep, double initialMargin);
CHARTDIR_DLLEXPORT void CArrayMath_selectStartOfDay(ArrayMathInternal *ptr, int majorTickStep, double initialMargin);
CHARTDIR_DLLEXPORT void CArrayMath_selectStartOfWeek(ArrayMathInternal *ptr, int majorTickStep, double initialMargin);
CHARTDIR_DLLEXPORT void CArrayMath_selectStartOfMonth(ArrayMathInternal *ptr, int majorTickStep, double initialMargin);
CHARTDIR_DLLEXPORT void CArrayMath_selectStartOfYear(ArrayMathInternal *ptr, int majorTickStep, double initialMargin);

CHARTDIR_DLLEXPORT void CArrayMath_trim(ArrayMathInternal *ptr, int startIndex, int len);
CHARTDIR_DLLEXPORT void CArrayMath_insert(ArrayMathInternal *ptr, const double *aData, int aLen, int insertPoint);
CHARTDIR_DLLEXPORT void CArrayMath_insert2(ArrayMathInternal *ptr, double c, int len, int insertPoint);
CHARTDIR_DLLEXPORT void CArrayMath_replace(ArrayMathInternal *ptr, double a, double b);

CHARTDIR_DLLEXPORT void CArrayMath_movAvg(ArrayMathInternal *ptr, int interval);
CHARTDIR_DLLEXPORT void CArrayMath_expAvg(ArrayMathInternal *ptr, double smoothingFactor);
CHARTDIR_DLLEXPORT void CArrayMath_movMed(ArrayMathInternal *ptr, int interval);
CHARTDIR_DLLEXPORT void CArrayMath_movPercentile(ArrayMathInternal *ptr, int interval, double percentile);
CHARTDIR_DLLEXPORT void CArrayMath_movMax(ArrayMathInternal *ptr, int interval);
CHARTDIR_DLLEXPORT void CArrayMath_movMin(ArrayMathInternal *ptr, int interval);
CHARTDIR_DLLEXPORT void CArrayMath_movStdDev(ArrayMathInternal *ptr, int interval);
CHARTDIR_DLLEXPORT void CArrayMath_movCorr(ArrayMathInternal *ptr, int interval, const double *bData, int bLen);
CHARTDIR_DLLEXPORT void CArrayMath_lowess(ArrayMathInternal *ptr, double smoothness, int iteration);
CHARTDIR_DLLEXPORT void CArrayMath_lowess2(ArrayMathInternal *ptr, const double *bData, int bLen, double smoothness, int iteration);

CHARTDIR_DLLEXPORT void CArrayMath_result(ArrayMathInternal *ptr, const double **ret, int *retLen);
CHARTDIR_DLLEXPORT double CArrayMath_max(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT double CArrayMath_min(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT double CArrayMath_avg(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT double CArrayMath_sum(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT double CArrayMath_med(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT double CArrayMath_percentile(ArrayMathInternal *ptr, double p);
CHARTDIR_DLLEXPORT int CArrayMath_maxIndex(ArrayMathInternal *ptr);
CHARTDIR_DLLEXPORT int CArrayMath_minIndex(ArrayMathInternal *ptr);


////////////////////////////////////////////////////////////////////////////////////////
//	rantable.h
////////////////////////////////////////////////////////////////////////////////////////
class RanTableInternal;
CHARTDIR_DLLEXPORT RanTableInternal *CRanTable_create(int seed, int noOfCols, int noOfRows);
CHARTDIR_DLLEXPORT void CRanTable_destroy(RanTableInternal *ptr);

CHARTDIR_DLLEXPORT void CRanTable_setCol(RanTableInternal *ptr, int colNo, double minValue, double maxValue);
CHARTDIR_DLLEXPORT void CRanTable_setCol2(RanTableInternal *ptr, int colNo, double startValue, double minDelta, double maxDelta,
	double lowerLimit, double upperLimit);
CHARTDIR_DLLEXPORT void CRanTable_setDateCol(RanTableInternal *ptr, int i, double startTime, double tickInc, bool weekDayOnly);
CHARTDIR_DLLEXPORT void CRanTable_setHLOCCols(RanTableInternal *ptr, int i, double startValue, double minDelta, double maxDelta,
	double lowerLimit, double upperLimit);
CHARTDIR_DLLEXPORT void CRanTable_getCol(RanTableInternal *ptr, int i, const double **ret, int *len);


}

#endif
